﻿using System;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Nikse.SubtitleEdit.Logic;

namespace Nikse.SubtitleEdit.Forms
{
    public partial class ChooseEncodingSave : Form
    {
        class ListViewSorter : System.Collections.IComparer
        {
            public int Compare(object o1, object o2)
            {
                var lvi1 = o1 as ListViewItem;
                var lvi2 = o2 as ListViewItem;
                if (lvi1 == null || lvi2 == null)
                    return 0;

                if (Descending)
                {
                    ListViewItem temp = lvi1;
                    lvi1 = lvi2;
                    lvi2 = temp;
                }

                if (IsNumber)
                {
                    int i1 = int.Parse(lvi1.SubItems[ColumnNumber].Text);
                    int i2 = int.Parse(lvi2.SubItems[ColumnNumber].Text);

                    if (i1 > i2)
                        return 1;
                    if (i1 == i2)
                        return 0;
                    return -1;
                }
                return string.Compare(lvi2.SubItems[ColumnNumber].Text, lvi1.SubItems[ColumnNumber].Text);
            }
            public int ColumnNumber { get; set; }
            public bool IsNumber { get; set; }
            public bool Descending { get; set; }
        }

        private Encoding _encoding;
        private int _selectedStartIndex = 0;

        public ChooseEncodingSave()
        {
            InitializeComponent();
            LanguageStructure.ChooseEncoding language = Configuration.Settings.Language.ChooseEncoding;
            Text = language.Title;
            listView1.Columns[0].Text = language.CodePage;
            listView1.Columns[1].Text = Configuration.Settings.Language.General.Name;
            listView1.Columns[2].Text = language.DisplayName;
            buttonOK.Text = Configuration.Settings.Language.General.OK;
            buttonCancel.Text = Configuration.Settings.Language.General.Cancel;
            FixLargeFonts();

            Encoding encoding = Encoding.UTF8;
            foreach (EncodingInfo ei in Encoding.GetEncodings())
            {
                var item = new ListViewItem(new[] { ei.CodePage.ToString(), ei.Name, ei.DisplayName });
                listView1.Items.Add(item);
                if (ei.CodePage == encoding.CodePage)
                {
                    item.Selected = true;
                    _selectedStartIndex = listView1.Items.Count-1;
                }
            }

            listView1.ListViewItemSorter = new ListViewSorter { ColumnNumber = 0, IsNumber = true };
        }

        private void FixLargeFonts()
        {
            Graphics graphics = this.CreateGraphics();
            SizeF textSize = graphics.MeasureString(buttonOK.Text, this.Font);
            if (textSize.Height > buttonOK.Height - 4)
            {
                int newButtonHeight = (int)(textSize.Height + 7 + 0.5);
                Utilities.SetButtonHeight(this, newButtonHeight, 1);
            }
        }

        private void FormChooseEncoding_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Escape)
                DialogResult = DialogResult.Cancel;
        }

        internal Encoding GetEncoding()
        {
            return _encoding;
        }

        private void ButtonOkClick(object sender, EventArgs e)
        {
            if (listView1.SelectedItems.Count == 0)
                MessageBox.Show(Configuration.Settings.Language.ChooseEncoding.PleaseSelectAnEncoding);
            else
            {
                _encoding = Encoding.GetEncoding(int.Parse(listView1.SelectedItems[0].Text));
                DialogResult = DialogResult.OK;
            }
        }

        private void listView1_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            var sorter = (ListViewSorter)listView1.ListViewItemSorter;

            if (e.Column == sorter.ColumnNumber)
            {
                sorter.Descending = !sorter.Descending; // inverse sort direction
            }
            else
            {
                sorter.ColumnNumber = e.Column;
                sorter.Descending = false;
                sorter.IsNumber = e.Column == 0; // only index 1 is numeric
            }
            listView1.Sort();
        }

        private void ChooseEncodingSave_Shown(object sender, EventArgs e)
        {
            listView1.EnsureVisible(_selectedStartIndex);
        }

    }
}
