/* 
Subtitles Workgroup: A subtitles creation software with network collaboration suppport
Copyright (C) 2007 smalleye

This program is free software; you can redistribute it and/or modify it under the terms of the 
 * GNU General Public License as published by the Free Software Foundation; either version 2 of 
 * the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; 
 * if not, write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using System.Collections.Specialized;

namespace Nikse.SubtitleEdit.Logic.Networking
{
    public class NetworkMessage:NetworkOperation
    {
        private StringCollection _data;

        public StringCollection ToStream()
        {
            StringCollection result = new StringCollection();
            base.AddParameter(_data.Count.ToString(), false);
            result.Add(base.Line);

            foreach (string line in _data)
                result.Add(line);
            return result;
        }

        public NetworkMessage(string line)
            : base()
        {
            base.Line = line;
            _data = new StringCollection();
            toStreamDlg = ToStream;
        }

        public NetworkMessage(string type, long newtrID)
            : base()
        {
            Name = "MSG";
            trID = newtrID;
            AddParameter(type, true);
            _data = new StringCollection();
            toStreamDlg = ToStream;
        }

        public NetworkMessage(string type, long newtrID, string parameters)
            : this(type, newtrID)
        {
            AddParameter(parameters, false);
        }

        public void SetHeader(string param, string value)
        {
            bool found = false;
            int index = 0;

            while ((index < _data.Count) && (_data[index] != "") && (!found))
            {
                string myline = _data[index];
                found = myline.Substring(0, param.Length) == param;
                index++;
            }
            if (found) _data.RemoveAt(index - 1);

            if (_data.Count == 0)
            {
                _data.Add(param + ": " + value);
                _data.Add("");
            }
            else
            {
                index = 0;
                bool rnfound = false;
                while ((!rnfound) && (index < _data.Count))
                {
                    rnfound = _data[index] == "";
                    index++;
                }
                index--;
                _data.Insert(index, param + ": " + value);
            }
        }

        public void AddBody(string newline)
        {
            if (_data.Count == 0)
            {
                _data.Add("");
                _data.Add(newline);
            }
            else
            {
                _data.Add(newline);
            }
        }

        public StringCollection GetBody()
        {
            StringCollection result = new StringCollection();
            int index = 0;
            bool rnfound = false;
            while (!rnfound && index < _data.Count)
            {
                rnfound = _data[index] == "";
                index++;
            }

            for (int c = index; c < _data.Count; c++)
                result.Add(_data[c]);
            return result;
        }

        public StringCollection GetHeaders()
        {
            StringCollection result = new StringCollection();
            int index = 0;
            bool rnfound = false;
            while (!rnfound && index < _data.Count)
            {
                rnfound = _data[index] == "";
                index++;
            }

            for (int c = 0; c < index-2; c++)
                result.Add(_data[c]);
            return result;
        }

        public string GetHeader(string param)
        {
            bool found = false;
            int index = 0;
            while (!found && index < _data.Count && _data[index] != string.Empty)
            {
                found = _data[index].Substring(0, param.Length) == param;
                index++;
            }
            index--;

            if (found)
            {
                string matchline = _data[index];
                return matchline.Substring(param.Length + 2, matchline.Length - param.Length - 2);
            }
            return null;
        }

        public void AddData(string ldata)
        {
            _data.Add(ldata);
        }

        public string GetKind()
        {
            return GetParameter(2);
        }

    }
}
