/* 
Subtitles Workgroup: A subtitles creation software with network collaboration suppport
Copyright (C) 2007 smalleye

This program is free software; you can redistribute it and/or modify it under the terms of the 
 * GNU General Public License as published by the Free Software Foundation; either version 2 of 
 * the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; 
 * if not, write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using System;
using System.Collections.Specialized;
using System.Text;
using System.IO;
using System.Web;


namespace Nikse.SubtitleEdit.Logic.Networking
{
    public class NetworkOperation
    {
        public delegate StringCollection dlg_toStream();
        public dlg_toStream toStreamDlg;

        private string _commandline;

        public void SetParameter(int index, string value)
        {
            string[] parameters = _commandline.Split();
            if (index < parameters.Length)
            {
                _commandline = "";
                for (int c = 0; c < parameters.Length; c++)
                    if (c != index)
                        _commandline += parameters[c]+ " ";
                    else
                        _commandline += value+ " ";
                _commandline = _commandline.Substring(0, _commandline.Length - 1);
            }
            else
            {
                _commandline += " " + value;
            }
        }

        public string GetParameter(int index)
        {
            string[] parameters = _commandline.Split();
            if (index < parameters.Length)
                return parameters[index];
            return null;            
        }

        public string Line
        {
            set
            {
                _commandline = value;
            }
            get
            {
                return _commandline;
            }
        }

        public long trID
        {
            set
            {
                SetParameter(1, Convert.ToString(value));

            }
            get
            {
                return Convert.ToInt64(GetParameter(1));
            }
        }
        
        public string Name
        {
            set
            {
                SetParameter(0, value);
            }
            get
            {
                return GetParameter(0);
            }
        }

        public string[] Parameters
        {
            get
            {
                string[] allline = _commandline.Split();
                string[] result = new string[allline.Length - 2];
                for (int c = 2; c < allline.Length; c++)
                    result[c - 2] = allline[c];
                return result;
            }
            set
            {
                _commandline = GetParameter(0) + " " + GetParameter(1);
                foreach (string p in value)
                    _commandline += " " + p;
            }
        }

        public NetworkOperation()
        {
            _commandline = "";
        }

        public NetworkOperation(string newName, long newtrID)
            : this()
        {
            Name = newName;
            trID = newtrID;
        }

        public NetworkOperation(string newLine)
        {
            _commandline = newLine;
        }

        public NetworkOperation(string newName, long newtrID, string newParameters)
            :this(newName,newtrID) 
        {
            SetParameter(2, newParameters);

        }

        public NetworkOperation(string newname, long newtrID, string[] newParameters)
            : this(newname, newtrID)
        {
            foreach (string str in newParameters)
                _commandline += " " + str;
        }

        public int CountParameters()
        {
            string[] division = _commandline.Split(' ');
            return division.Length;
        }

        public void AddParameter(string value, bool encode)
        {
            string myvalue;
            if (encode)
                myvalue = HttpUtility.UrlEncode(value);
            else
                myvalue = value;
            _commandline += " " + myvalue;    
        }

        public string GetDecodedParameter(int index)
        {
            return HttpUtility.UrlDecode(GetParameter(index));
        }

        public string ParametersLine()
        {
            string[] pars = _commandline.Split(' ');
            string result="";
            if (pars.Length > 2)
            {
                for (int c = 2; c < pars.Length; c++)
                    result += pars[c] + " ";
                result = result.Substring(0, result.Length - 1);
                return result;
            }
            return null;         
        }

        public void ClearParameters()
        {
            string[] pars = _commandline.Split(' ');
            _commandline = pars[0] + " " + pars[1];
        }


    }
}
