/* 
Subtitles Workgroup: A subtitles creation software with network collaboration suppport
Copyright (C) 2007 smalleye

This program is free software; you can redistribute it and/or modify it under the terms of the 
 * GNU General Public License as published by the Free Software Foundation; either version 2 of 
 * the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; 
 * if not, write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using System.Collections;
using System.Net;
using System.Net.Sockets;
using System.Threading;

namespace Nikse.SubtitleEdit.Logic.Networking
{
    public class NetworkServer
    {

        private class clientList : ArrayList
        {
            public void Add(NetworkConnection connection)
            {
                base.Add(connection);
            }

            public new NetworkConnection this[int index]
            {
                get
                {
                    return (NetworkConnection)base[index];
                }
                set
                {
                    base[index] = value;
                }
            }

            public NetworkConnection GetId(int connid)
            {
                bool found = false;
                int c = 0;

                while (!found && c < Count)
                {
                    found = this[c].ConnectionId == connid;
                    c++;
                }

                if (found)
                    return this[c - 1];
                else
                    return null;
            }

            public void RemoveConnection(int connectionId)
            {
                NetworkConnection sconn = GetId(connectionId);
                base.Remove(sconn);
            }
        }

        public delegate void dlgNewClient(int id);
        
        private int _ccount = 1;
        private TcpListener _listener;
        private clientList _clients;
        public event NetworkConnection.dlgClientDisconnect OnClientDisconnect;
        public event NetworkConnection.dlgReceive OnReceive;
        public event dlgNewClient OnNewClient;
        private Thread _acceptThread;
       
        public NetworkServer(int port)
        {
            _clients = new clientList();
            string strHostName = Dns.GetHostName ();
            IPHostEntry ipEntry = Dns.GetHostEntry(strHostName);
            IPAddress [] addr = ipEntry.AddressList;
            IPAddress ipad = addr[0];
//            _listener = new TcpListener(ipad, port);
            IPAddress localAddress = IPAddress.Parse("0.0.0.0");
            _listener = new TcpListener(localAddress, port);
            _listener.Start();
            _acceptThread = new Thread(new ThreadStart(AcceptTh));
            _acceptThread.Start();
        }

        public void ClientDisconnect(int id)
        {
            if (OnClientDisconnect != null) 
                OnClientDisconnect(id);
            _clients.RemoveConnection(id);

        }

        private void Receive(int id, NetworkOperation op)
        {
            if (OnReceive != null) 
                OnReceive(id, op);
        }

        private void AcceptTh()
        {
            try
            {
                while (true)
                {
                    TcpClient newclient = _listener.AcceptTcpClient();
                    NetworkConnection newconn = new NetworkConnection(newclient, _ccount);
                    _ccount++;
                    newconn.OnClientDisconnect += ClientDisconnect;
                    newconn.OnReceive += OnReceive;
                    _clients.Add(newconn);
                    if (OnNewClient != null) 
                        OnNewClient(newconn.ConnectionId);
                }
            }
            catch 
            {
            }            
        }

        public void Stop()
        {
            _acceptThread.Abort();
            _acceptThread = null;
            while (_clients.Count > 0) _clients[0].Disconnect();
            _listener.Stop();
            _clients.Clear();
        }

        public NetworkOperation Send(int id, NetworkOperation operation, bool wait)
        {
            NetworkConnection connection;
            if ((connection = _clients.GetId(id)) != null)
                return connection.Send(operation, wait);
            return null;
        }

        public void sendToAll(NetworkOperation operation)
        {
            foreach (NetworkConnection connection in _clients)
                connection.Send(operation, false);
        }

        public void sendToAllExceptOne(int id, NetworkOperation operation)
        {
            foreach (NetworkConnection connection in _clients)
            {
                if (connection.ConnectionId != id)
                    connection.Send(operation, false);
            }
        }

        public void DisconnectClient(int id)
        {
            NetworkConnection conn = _clients.GetId(id);
            if (conn != null) conn.Disconnect();
        }
    }
}
