﻿using System;
using System.Collections.Specialized;
using System.Text;

namespace Nikse.SubtitleEdit.Logic.Networking
{
    public class SwgClient
    {
        public UserList users;
        
        public delegate void dlgConnected(string nick);
        public event dlgConnected onConnected;
        public delegate void dlgNewPartner(int id, string nick);
        public event dlgNewPartner onNewPartner;
        public delegate void dlgConn(int id);
        public event dlgConn onByePartner;
        public event dlgConn onShareControls;
        public delegate void dlgFile(string filename, long size);
        public event dlgFile onSetFile;
        public delegate void dlgAction();
        public event dlgAction onRequestFile;
        public delegate void dlgSequence(int id, int pos, int starttime, int endtime, string text, bool initial);
        public event dlgSequence onSequence;
        public delegate void dlgTime(int id, int pos, int ntime);
        public event dlgTime onStartTime;
        public event dlgTime onEndTime;
        public delegate void dlgText(int id, int pos, string text, bool translated);
        public event dlgText onText;
        public delegate void dlgChat(int id, string text);
        public event dlgChat onChat;
        public delegate void dlgInitial(int numseq);
        public event dlgInitial onInitial;
        public delegate void dlgDelete(int id, int index);
        public event dlgDelete onDelete;
        public delegate void dlgPlay(int id, bool on);
        public event dlgPlay onPlay;
        public delegate void dlgPosition(int id, int pos);
        public event dlgPosition onPosition;
        public event dlgPosition onTake;


        public enum msgtypes
        {
            NULL=0,
            SEQ=1,
            TEX=2,
            CHA=3,
        }

        public NetworkClient Client;
        public String Nick;
        

        public event NetworkConnection.dlgClientDisconnect onDisconnect;

        private Nikse.SubtitleEdit.Forms.Networking receptor;

        public SwgClient(Nikse.SubtitleEdit.Forms.Networking newreceptor, string host, int port)
        {
            receptor = newreceptor;
            users = new UserList();
            Client = new NetworkClient(host, port, Receive, Disconnection);
            while (!Client.Connected) ;            
        }

        public msgtypes MessageTypeToEnum(NetworkMessage msg)
        {
            if (msg.GetKind() == "SEQ") return msgtypes.SEQ;
            if (msg.GetKind() == "CHA") return msgtypes.CHA;
            if (msg.GetKind() == "TEX") return msgtypes.TEX;

            return msgtypes.NULL;
        }

        public void Receive(int id, NetworkOperation op)
        {
            switch (SwgServer.opToEnum(op))
            {
                case (SwgServer.Operations.VER): VER(id, op); break;
                case (SwgServer.Operations.USR): USR(id, op); break;
                case (SwgServer.Operations.JOI): JOI(id, op); break;
                case (SwgServer.Operations.LST): LST(id, op); break;
                case (SwgServer.Operations.BYE): BYE(id, op); break;
                case (SwgServer.Operations.FIL): FIL(id, op); break;
                case (SwgServer.Operations.SST): SST(id, op); break;
                case (SwgServer.Operations.SET): SET(id, op); break;
                case (SwgServer.Operations.INI): INI(id, op); break;
                case (SwgServer.Operations.DEL): DEL(id, op); break;
                case (SwgServer.Operations.SHA): SHA(id, op); break;
                case SwgServer.Operations.PLA: PLA(id, op); break;
                case SwgServer.Operations.POS: POS(id, op); break;
                case SwgServer.Operations.TAK: TAK(id, op); break;

                //mensajes
                case (SwgServer.Operations.MSG):
                    switch (MessageTypeToEnum((NetworkMessage) op))
                    {
                        case msgtypes.SEQ: msgSEQ(id, (NetworkMessage) op);break;
                        case msgtypes.TEX: msgTEX(id, (NetworkMessage)op); break;
                        case msgtypes.CHA: msgCHA(id, (NetworkMessage)op); break;
                    }
                    break;                  
            }
        }

        private void SHA(int id, NetworkOperation op)
        {
            int myid = Convert.ToInt32(op.GetParameter(2));
            if (onShareControls != null)
                receptor.Invoke(new dlgConn(receptor.client_onShareControls), new object[] { myid });
        }

        private void INI(int id, NetworkOperation op)
        {
            int max = Convert.ToInt32(op.GetParameter(2));
            if (onInitial != null)
                receptor.Invoke(new dlgInitial(receptor.client_onInitial), new object[] { max });
        }

        public void Start(string nick)
        {
            this.Nick = nick;
            NetworkCommand VER = new NetworkCommand("VER", 0, "SWP1");
            Client.Send(VER, false);
        }

        private void VER(int id, NetworkOperation op)
        {
            if (op.ParametersLine().IndexOf("SWP1") > -1)
            {
                NetworkCommand USR = new NetworkCommand("USR", 0);
                USR.AddParameter(Nick, true);
                Client.Send(USR, false);
            }
            else
                Disconnect();
        }

        private void USR(int id, NetworkOperation op)
        {
            if (onConnected != null)
            {
                receptor.Invoke(new dlgConnected(receptor.client_onConnected), new object[] { Nick });
            }
            if (op.CountParameters() > 4)
            {
                int myid = Convert.ToInt32(op.GetParameter(3));
                Client.SetId(myid);
                string filename = op.GetDecodedParameter(4);
                long filesize = Convert.ToInt64(op.GetParameter(5));
                if (onSetFile != null)
                    receptor.Invoke(new dlgFile(receptor.client_onSetFile), new object[] { filename, filesize });
            }
            else
            {
                int myid = Convert.ToInt32(op.GetParameter(3));
                Client.SetId(myid);
                if (onRequestFile != null)
                    receptor.Invoke(new dlgAction(receptor.client_onRequestFile));
            }
        }

        private void JOI(int id, NetworkOperation op)
        {
            int connid = Convert.ToInt32(op.GetDecodedParameter(2));
            string newnick = op.GetDecodedParameter(3);
            users.Add(newnick, connid);
            op.ClearParameters();
            op.AddParameter("OK", true);
            Client.Send(op, false);
            if (onNewPartner != null)
                receptor.Invoke(new dlgNewPartner(receptor.client_onNewPartner), new object[] { connid, newnick });
        }

        private void LST(int id, NetworkOperation op)
        {
            int connid = Convert.ToInt32(op.GetDecodedParameter(2));
            string newnick = op.GetDecodedParameter(3);
            users.Add(newnick, connid);
            op.ClearParameters();
            op.AddParameter("OK", true);
            Client.Send(op, false);
            if (onNewPartner != null) 
                receptor.Invoke(new dlgNewPartner(receptor.client_onNewPartner), new object[] {connid, newnick});
        }

        private void BYE(int id, NetworkOperation op)
        {
            int connid = Convert.ToInt32(op.GetDecodedParameter(2));
            if (onByePartner != null)
                receptor.Invoke(new dlgConn(receptor.client_onByePartner), new object[] { connid });
            users.delUser(connid);         
        }

        public void Disconnection(int id)
        {
            if (onDisconnect != null) 
                onDisconnect(id);
        }

        public void requestFile(string filename, long size)
        {
            NetworkCommand FIL = new NetworkCommand("FIL", 0);
            FIL.AddParameter(filename, true);
            FIL.AddParameter(size.ToString(), false);
            Client.Send(FIL, false);
        }

        private void FIL(int id, NetworkOperation op)
        {
            string filename = op.GetDecodedParameter(2);
            long filesize = Convert.ToInt64(op.GetParameter(3));

            if (onSetFile != null) onSetFile(filename, filesize);
            op.ClearParameters();
            op.AddParameter("OK", false);
            Client.Send(op, false);
        }

        public static NetworkMessage createSEQMsg(int myid, int pos, int starttime, int endtime, StringCollection text, bool initial)
        {
            NetworkMessage SEQ = new NetworkMessage("SEQ", 0);
            SEQ.AddParameter(myid.ToString(), false);
            SEQ.AddParameter(pos.ToString(), false);
            if (initial)
                SEQ.AddParameter("Y", false);
            else
                SEQ.AddParameter("N", false);
            SEQ.SetHeader("Start-time", starttime.ToString());
            SEQ.SetHeader("End-time", endtime.ToString());
            foreach (string line in text)
                SEQ.AddBody(line);
            return SEQ;
        }

        public void SendSequence(int pos, int starttime, int endtime, string text, bool initial)
        {
            System.Collections.Specialized.StringCollection sc = new System.Collections.Specialized.StringCollection();
            foreach (string line in text.Split(Environment.NewLine.ToCharArray(), StringSplitOptions.RemoveEmptyEntries))
                sc.Add(line);

            NetworkMessage SEQ = createSEQMsg(Client.GetId(), pos, starttime, endtime, sc, initial);
            Client.Send(SEQ, false);
        }

        private void msgSEQ(int id, NetworkMessage op)
        {
            int myid = Convert.ToInt32(op.GetParameter(3));
            int seqnumber = Convert.ToInt32(op.GetParameter(4));
            bool initial = op.GetParameter(5) == "Y";
            int startime = Convert.ToInt32(op.GetHeader("Start-time"));
            int endtime = Convert.ToInt32(op.GetHeader("End-time"));

            StringCollection text = op.GetBody();
            StringBuilder sb = new StringBuilder();
            foreach (string line in text)
                sb.AppendLine(line);


            if (onSequence != null)
            {
                try
                {
                    receptor.Invoke(new dlgSequence(receptor.client_onSequence), new object[] { myid, seqnumber, startime, endtime, sb.ToString(), initial });
                }
                catch
                {
                }
            }
        }

        public void SetStartTime(int pos, int stime)
        {
            NetworkCommand SST = new NetworkCommand("SST", 0);
            SST.AddParameter(Client.GetId().ToString(), false);
            SST.AddParameter(pos.ToString(), false);
            SST.AddParameter(stime.ToString(), false);
            Client.Send(SST, false);
        }

        public void SetEndTime(int pos, int stime)
        {
            NetworkCommand SET = new NetworkCommand("SET", 0);

            SET.AddParameter(Client.GetId().ToString(), false);
            SET.AddParameter(pos.ToString(), false);
            SET.AddParameter(stime.ToString(), false);
            Client.Send(SET, false);
        }

        private void SST(int id, NetworkOperation op)
        {
            int myid = Convert.ToInt32(op.GetParameter(2));
            int pos = Convert.ToInt32(op.GetParameter(3));
            int time = Convert.ToInt32(op.GetParameter(4));

            if (onStartTime != null)
                receptor.Invoke(new dlgTime(receptor.client_onStartTime), new object[] { myid, pos, time });
        }

        private void SET(int id, NetworkOperation op)
        {
            int pos = Convert.ToInt32(op.GetParameter(3));
            int time = Convert.ToInt32(op.GetParameter(4));
            int myid = Convert.ToInt32(op.GetParameter(2));

            if (onEndTime != null)
                receptor.Invoke(new dlgTime(receptor.client_onEndTime), new object[] { myid, pos, time });
        }


        public static NetworkMessage CreateTEXMessage(int myid, int pos, StringCollection text, bool translated)
        {
            NetworkMessage op = new NetworkMessage("TEX", 0);
            op.AddParameter(myid.ToString(), false);
            op.AddParameter(pos.ToString(), false);
            if (translated)
                op.AddParameter("T", false);
            else
                op.AddParameter("0", false);

            foreach (string line in text)
                op.AddBody(line);

            return op;
        }

        public void ChangeText(int pos, StringCollection text, bool translated)
        {
            NetworkMessage op = CreateTEXMessage(Client.GetId(), pos, text, translated);
            Client.Send(op, false);
        }

        private void msgTEX(int id, NetworkMessage op)
        {
            StringCollection text = op.GetBody();
            int myid = Convert.ToInt32(op.GetParameter(3));
            int pos = Convert.ToInt32(op.GetParameter(4));
            bool translated = op.GetParameter(5) == "T";

            StringBuilder sb = new StringBuilder();
            foreach (string line in text)
                sb.AppendLine(line);

            if (onText != null)
                receptor.Invoke(new dlgText(receptor.client_onText), new object[] { myid, pos, sb.ToString(),translated });
        }

        public void Disconnect()
        {
            if (Client!=null)
                Client.Disconnect();
            if (users!=null)
             users.Clear();
            users = null;
            Client = null;
        }

        public void Ping()
        {
            Client.Send(new NetworkCommand("PNG", 0), false);
        }

        public void SendChat(string text)
        {
            NetworkMessage op = new NetworkMessage("CHA", 0);
            op.AddParameter(Client.GetId().ToString(), false);
            op.AddBody(text);
            Client.Send(op, false);
        }

        public void msgCHA(int id, NetworkMessage op)
        {
            int myid = Convert.ToInt32(op.GetParameter(3));
            StringCollection text = op.GetBody();

            StringBuilder sb = new StringBuilder();
            foreach (string line in text)
                sb.AppendLine(line);

            if (onChat != null)
                receptor.Invoke(new dlgChat(receptor.client_onChat), new object[] { myid, sb.ToString() });
        }

        public void Request()
        {
            NetworkCommand op = new NetworkCommand("REQ", 0);
            Client.Send(op, false);
        }

        public void DeleteSequence(int pos)
        {
            NetworkCommand op = new NetworkCommand("DEL", 0);
            op.AddParameter(Client.GetId().ToString(), false);
            op.AddParameter(pos.ToString(), false);
            Client.Send(op, false);
        }

        public void DEL(int id, NetworkOperation op)
        {
            int myid = Convert.ToInt32(op.GetParameter(2));
            int pos = Convert.ToInt32(op.GetParameter(3));

            if (onDelete != null)
                receptor.Invoke(new dlgDelete(receptor.client_onDelete), new object[] { myid, pos });

        }

        public void shareControls()
        {
            NetworkCommand op = new NetworkCommand("SHA", 0);
            op.AddParameter(Client.GetId().ToString(), false);
            Client.Send(op, false);
        }

        public void Play(bool on)
        {
            NetworkCommand op = new NetworkCommand("PLA", 0);
            op.AddParameter(Client.GetId().ToString(), false);
            if (on) op.AddParameter("ON", false);
            else op.AddParameter("OFF", false);

            Client.Send(op, false);
        }

        public void PLA(int id, NetworkOperation op)
        {
            int myid = Convert.ToInt32(op.GetParameter(2));
            bool on = op.GetParameter(3) == "ON";

            if (onPlay!=null)
                receptor.Invoke(new dlgPlay(receptor.client_onPlay), new object[]{myid, on});

        }

        public void Position(int newpos)
        {
            NetworkCommand op = new NetworkCommand("POS", 0);
            op.AddParameter(Client.GetId().ToString(), false);
            op.AddParameter(newpos.ToString(), false);

            Client.Send(op, false);
        }

        public void POS(int id, NetworkOperation op)
        {
            int myid = Convert.ToInt32(op.GetParameter(2));
            int mypos = Convert.ToInt32(op.GetParameter(3));

            if (onPosition!=null)
                receptor.Invoke(new dlgPosition(receptor.client_onPosition), new object[] {myid, mypos});
        }

        public void TakeUser(string nick, int seqnumber)
        {
            NetworkCommand op = new NetworkCommand("TAK", 0);
            op.AddParameter(Client.GetId().ToString(), false);
            int userid = users.searchNick(nick).ConnectionId;
            op.AddParameter(userid.ToString(), false);
            op.AddParameter(seqnumber.ToString(), false);

            Client.Send(op, false);
        }

        public void TAK(int id, NetworkOperation op)
        {
            int myid = Convert.ToInt32(op.GetParameter(2));
            int mypos = Convert.ToInt32(op.GetParameter(4));

            if (onTake != null)
                receptor.Invoke(new dlgPosition(receptor.client_onTake), new object[] { myid, mypos });
        }

    }
}
